<?xml version="1.0" encoding="UTF-8"?>
<!-- XÖV-Profil 1.5.0 - XÖV-Übersetzungsanweisungen - XML Schema-Teil -->
<!-- $Id: xoev-profil.helpers.xsl 10192 2019-02-06 14:38:33Z mkuhlmann $ -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xmi="http://www.omg.org/spec/XMI/20131001" xmlns:xgen="http://www.xoev.de/de/xgenerator/framework/1/library" xmlns:xoev="http://www.xoev.de/de/xoev-profil/1_5_1" xmlns:uml="http://www.eclipse.org/uml2/5.0.0/UML" xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" version="3.0" exclude-result-prefixes="xsl xmi xgen xoev uml xd">

    <!-- **************************************************************************************************************************************** -->
    <!-- ***** Kontext: Beliebig **************************************************************************************************************** -->
    <!-- **************************************************************************************************************************************** -->

    <xd:doc>
        <xd:desc>Key zur Ermittlung der Menge von Dependencies, in denen die angegebene ID als client fungiert, es sich also um ein abhängiges Element handelt.</xd:desc>
    </xd:doc>
    <!-- Implementierungshinweis: Anstelle von uml:is-Dependeny wird hier bewusst die Definition direkt eingefügt, da Saxon-HE 
         dies im Rahmen der Optimierung nicht tut und der Profiler gezeigt hat, dass dies hier ein Performance-Hotspot ist. -->
    <!-- TODO: An dieser Stelle lieber zur Sicherheit uml:conforms-to-Dependency benutzen - oder ggf. uml:Usage direkt
         unten mit aufnehmen -->
    <xsl:key name="xoev:dependency-by-client" match="packagedElement[@xmi:type = 'uml:Dependency']" use="@client"/>
    <xsl:function name="xoev:dependencies-by-client" as="element()*">
        <xsl:param name="element" as="element()"/>
        <xsl:sequence select="key('xoev:dependency-by-client', $element/@xmi:id, root($element))"/>
    </xsl:function>

    <xsl:function name="xoev:resolve-base" as="element()*">
        <xsl:param name="element-with-base" as="element()?"/>
        <xsl:sequence select="xgen:resolve-ids-or-hrefs($element-with-base/@base, $element-with-base/base)"/>
    </xsl:function>

    <xsl:function name="xoev:ids-to-elements" as="element()*">
        <xsl:param name="ids" as="xs:string*"/>
        <xsl:param name="root"/>
        <xsl:for-each select="$ids">
            <xsl:sequence select="xgen:safe-key('uml:_element-by-id', ., $root)"/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:allImportedAndIncludedPackageIDs" as="xs:string*">
        <xsl:param name="importedOrIncludesPackagesIDs" as="xs:string*"/>
        <xsl:param name="root"/>
        <xsl:variable name="nextStep" select="
                ($importedOrIncludesPackagesIDs,
                xoev:ids-to-elements($importedOrIncludesPackagesIDs, $root)/xoev:dependencies-by-client(.)[xoev:xsdImport(.) or xoev:xsdInclude(.)]/uml:resolve-supplier(.)/@xmi:id)" as="xs:string*"/>
        <xsl:choose>
            <xsl:when test="count($importedOrIncludesPackagesIDs) = count(distinct-values($nextStep))">
                <xsl:sequence select="$importedOrIncludesPackagesIDs"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="xoev:allImportedAndIncludedPackageIDs(distinct-values($nextStep), $root)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:key name="xoev:type-by-name" match="packagedElement | nestedClassifier" use="@name"/>
    <xsl:function name="xoev:type-by-name" as="element()*">
        <xsl:param name="name"/>
        <xsl:param name="context" as="node()"/>
        <xsl:sequence select="key('xoev:type-by-name', $name, $context)"/>
    </xsl:function>

    <xsl:key name="xoev:code-by-uri" match="packagedElement | nestedClassifier" use="ownedAttribute[@name = 'listURI']/(defaultValue/@value, defaultValue/body)[1]"/>
    <xsl:function name="xoev:code-by-uri" as="element()*">
        <xsl:param name="uri"/>
        <xsl:param name="context" as="node()"/>
        <xsl:sequence select="key('xoev:code-by-uri', $uri, root($context))"/>
    </xsl:function>

    <xsl:key name="xoev:codelist-by-uri" match="packagedElement | nestedClassifier" use="xoev:xoevCodeliste(.)/@kennung"/>
    <xsl:function name="xoev:codelist-by-uri" as="element()*">
        <xsl:param name="uri"/>
        <xsl:param name="context" as="node()"/>
        <xsl:sequence select="key('xoev:codelist-by-uri', $uri, root($context))"/>
    </xsl:function>

    <xsl:function name="xoev:getNamespace" as="xs:string?">
        <xsl:param name="element" as="element()"/>
        <xsl:sequence select="($element/ancestor-or-self::packagedElement/xoev:xsdSchema(.)/@namespace, $element/ancestor-or-self::packagedElement/xoev:xsdXModel(.)/@namespace)[1]"/>
    </xsl:function>

    <xd:doc>
        <xd:desc>Das eigene xsdXModel-Modell. Dieses entspricht dem (einzigen) xsdXModel-Modell, das außerhalb des Pakets 'Externe Modelle' vorliegt. Prüfanweisungen stellen sicher, dass genau ein xsdXModel-Modell auf dieser Ebene vorliegt.</xd:desc>
    </xd:doc>
    <xsl:variable name="xoev:my-xmodel" as="element(packagedElement)">
        <!-- [1] für den Fall, dass mehr als ein xsdXModel auf oberster Ebene existiert. Es soll kein XSLT-Fehler auftreten. Prüfanweisung fängt diesen Fall ab. -->
        <xsl:sequence select="$uml:root-model/packagedElement[xoev:xsdXModel(.)][1]"/>
    </xsl:variable>

    <xsl:function name="xoev:isReferenceableByName" as="xs:boolean">
        <xsl:param name="type" as="element()"/>
        <xsl:sequence select="$type/parent::packagedElement/@name = 'W3C Data Types' or xoev:xsdNamedType($type)"/>
    </xsl:function>

    <!-- **************************************************************************************************************************************** -->
    <!-- ***** Kontext: Datentypen und globale Elemente ***************************************************************************************** -->
    <!-- **************************************************************************************************************************************** -->

    <xsl:function name="xoev:lowerBound" as="xs:integer">
        <xsl:param name="property" as="element()"/>
        <xsl:choose>
            <xsl:when test="$property/lowerValue/@value = 1 or not($property/lowerValue)">1</xsl:when>
            <xsl:when test="$property/lowerValue and not($property/lowerValue/@value)">0</xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$property/lowerValue/@value"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="xoev:upperBound" as="xs:integer">
        <xsl:param name="property" as="element()"/>
        <xsl:choose>
            <xsl:when test="$property/upperValue/@value = '*'">-1</xsl:when>
            <xsl:when test="$property/upperValue/@value = '1' or not($property/upperValue)">1</xsl:when>
            <xsl:when test="$property/upperValue and not($property/upperValue/@value)">0</xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$property/upperValue/@value"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="xoev:allParents" as="element()*">
        <xsl:param name="class" as="element()"/>
        <xsl:for-each select="$class/generalization/uml:resolve-general(.)">
            <xsl:sequence select="."/>
            <xsl:sequence select="xoev:allParents(.)"/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:getDuplicateStrings" as="xs:string*">
        <xsl:param name="seq" as="xs:string*"/>
        <xsl:for-each select="$seq[index-of($seq, .)[2]]">
            <xsl:sequence select="."/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:allElementNamesIncludingRefsAndGroups" as="xs:string*">
        <xsl:param name="class" as="element()"/>
        <xsl:for-each select="xoev:allElements($class)">
            <xsl:choose>
                <xsl:when test="xoev:xsdElement(.)/@isRef and uml:resolve-type(.)[xoev:xsdGroup(.)]">
                    <xsl:sequence select="xoev:allElementNamesIncludingRefsAndGroups(uml:resolve-type(.))"/>
                </xsl:when>
                <xsl:when test="xoev:xsdElement(.)/@isRef and uml:resolve-type(.)[xoev:xsdGlobalElement(.)]">
                    <xsl:sequence select="uml:resolve-type(.)/@name"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:sequence select="@name"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:allElements" as="element()*">
        <xsl:param name="class" as="element()"/>
        <xsl:for-each select="$class/generalization[not(xoev:xsdRestriction(.))]/uml:resolve-general(.)">
            <xsl:sequence select="xoev:allElements(.)"/>
        </xsl:for-each>
        <xsl:sequence select="xoev:elements($class)"/>
    </xsl:function>

    <xsl:function name="xoev:allAttributes" as="element()*">
        <xsl:param name="class" as="element()"/>
        <xsl:sequence select="xoev:attributes($class)"/>
        <xsl:for-each select="$class/generalization[not(xoev:xsdRestriction(.))]/uml:resolve-general(.)">
            <xsl:sequence select="xoev:allAttributes(.)"/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:attributes" as="element()*">
        <xsl:param name="class" as="element()"/>
        <xsl:for-each select="$class/ownedAttribute[xoev:xsdAttribute(.)]">
            <xsl:sort select="@name"/>
            <xsl:sequence select="."/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:elements" as="element()*">
        <xsl:param name="class" as="element()"/>
        <xsl:for-each select="$class/ownedAttribute[xoev:xsdElement(.) or xoev:xsdLocalStructure(.)]">
            <xsl:sort select="number((xoev:xsdElement(.) | xoev:xsdLocalStructure(.))/@position)"/>
            <xsl:sequence select="."/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:getRealPosition" as="xs:integer">
        <xsl:param name="name" as="xs:string" required="yes"/>
        <xsl:param name="class" as="element()" required="yes"/>
        <xsl:choose>
            <xsl:when test="$name = $class/xoev:elements(.)/@name">
                <xsl:value-of select="
                        count(
                        $class/xoev:elements(.)[
                        number(xoev:xsdElement(.)/@position) le number($class/xoev:elements(.)[@name = $name]/xoev:xsdElement(.)/@position)]) +
                        count($class/generalization[not(xoev:xsdRestriction(.))]/uml:resolve-general(.)/xoev:allElements(.))"/>
            </xsl:when>
            <xsl:when test="$class/generalization[not(xoev:xsdRestriction(.))]">
                <xsl:value-of select="xoev:getRealPosition($name, $class/generalization/uml:resolve-general(.))"/>
            </xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="xoev:sortAndConcatForXMLRestriktionMitKorrekterElementenreihenfolge" as="xs:string*">
        <xsl:param name="class" as="element()" required="yes"/>
        <xsl:for-each select="$class/xoev:allElements(.)">
            <xsl:value-of select="
                    concat(codepoints-to-string(10), 'Position ', position(), ' in Elementsequenz: ', (if (xoev:xsdElement(.)/@isRef = 'true') then
                        uml:resolve-type(.)/@name
                    else
                        @name), if (position() = last()) then
                        codepoints-to-string(10)
                    else
                        '')"/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:qualified-type-name" as="xs:string">
        <!-- Typ, für den der qualifizierte Name ermittelt werden soll. -->
        <xsl:param name="type"/>
        <!-- Kontext (z. B. Element oder Typ), in dem der Typ (type) genutzt werden soll. -->
        <xsl:param name="referringContext" as="element()"/>
        <xsl:variable name="prefix">
            <xsl:choose>
                <xsl:when test="$type/parent::packagedElement/@name = 'W3C Data Types'">xs</xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="referringSchema" as="element()" select="$referringContext/ancestor::packagedElement[xoev:xsdSchema(.)][1]"/>
                    <xsl:variable name="typeSchema" as="element()?" select="$type/ancestor::packagedElement[xoev:xsdSchema(.)]"/>
                    <xsl:choose>
                        <xsl:when test="$typeSchema">
                            <xsl:variable name="include" select="xoev:package-with-imports-and-includes($referringSchema)/include[@id = $typeSchema/@xmi:id]" as="element(include)+"/>
                            <!-- Im Falle eines Mehrfach-Imports wird das erste import-Element genutzt. -->
                            <xsl:variable name="import" select="xoev:package-with-imports-and-includes($referringSchema)/import[@id = $typeSchema/@xmi:id][1]" as="element(import)?"/>
                            <xsl:choose>
                                <xsl:when test="$typeSchema/@xmi:id = $referringSchema/@xmi:id or $include">
                                    <xsl:value-of select="(xoev:xsdSchema($typeSchema)/@prefix, $typeSchema/ancestor::packagedElement/xoev:xsdXModel(.)/@prefix)[1]"/>
                                </xsl:when>
                                <xsl:when test="$import">
                                    <xsl:value-of select="$import/@prefix"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:message terminate="yes">Assertion failed: Could not determine namespace prefix for type <xsl:value-of select="$type/@name"/> in context of referring Schema <xsl:value-of select="$referringSchema/@name"/>.</xsl:message>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise></xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="concat($prefix, ':', $type/@name)"/>
    </xsl:function>


    <xsl:function name="xoev:topmost-parent" as="element()">
        <xsl:param name="class" as="element()"/>
        <xsl:choose>
            <!-- $class/generalization[1] anstelle von $class/generalization, damit kein Fehler bei Klassen mit mehr als einer ausgehenden Generalisierung
                 auftritt. Prüfanweisung NDR-28-ValideW3CXMLSchemaDefinitionen-EinfacheVererbung stellt sicher, dass Validierung zum error führt. -->
            <xsl:when test="$class/generalization">
                <xsl:sequence select="xoev:topmost-parent($class/generalization[1]/uml:resolve-general(.))"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="$class"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <!-- **************************************************************************************************************************************** -->
    <!-- ***** Kontext: Strings ******************************************************************************************************** -->
    <!-- **************************************************************************************************************************************** -->

    <xsl:function name="xoev:clean-text" as="xs:string">
        <xsl:param name="string"/>
        <xsl:value-of select="normalize-space(replace(replace(replace(replace($string, 
            '&lt;[^&lt;!][^&lt;]+[^-]&gt;', ''),
            '&amp;ndash;','-'),
            '&amp;([^;]+);', '$1'),
            '&lt;!--[^&gt;]*--&gt;',''))"/>
    </xsl:function>

    <!-- **************************************************************************************************************************************** -->
    <!-- ***** Kontext: xsdSchema-Pakete ******************************************************************************************************** -->
    <!-- **************************************************************************************************************************************** -->

    <xsl:function name="xoev:all-included-packages" as="element(packagedElement)*">
        <xsl:param name="package" as="element(packagedElement)"/>
        <xsl:for-each select="xoev:directly-included-packages($package)">
            <xsl:sequence select="."/>
            <xsl:sequence select="xoev:all-included-packages(.)"/>
        </xsl:for-each>
    </xsl:function>

    <xsl:function name="xoev:directly-included-packages" as="element(packagedElement)*">
        <xsl:param name="package" as="element(packagedElement)"/>
        <xsl:for-each select="xoev:dependencies-by-client($package)[xoev:xsdInclude(.)]">
            <xsl:sequence select="uml:resolve-supplier(.)"/>
        </xsl:for-each>
    </xsl:function>

    <xd:doc>
        <xd:desc>Die direkten xsdImports und xsdIncludes eines jeden xsdSchema-Pakets. Zum Zugriff wird die Funktion <xd:ref name="xoev:package-with-imports-and-includes" type="function"/> verwendet.</xd:desc>
    </xd:doc>
    <xsl:variable name="xoev:packages-with-imports-and-includes" as="document-node(element(packages-with-imports-and-includes))">
        <xsl:document>
            <packages-with-imports-and-includes>
                <xsl:for-each select="$uml:root-model//packagedElement[xoev:xsdSchema(.)]">
                    <package-with-imports-and-includes id="{@xmi:id}">
                        <xsl:for-each select="xoev:dependencies-by-client(.)[xoev:xsdImport(.) and uml:resolve-supplier(.)[xoev:xsdSchema(.)]]">
                            <xsl:variable name="imported-package" select="uml:resolve-supplier(.)[xoev:xsdSchema(.)]" as="element()"/>
                            <xsl:variable name="xmodel-of-imported-package" as="element()" select="$imported-package/ancestor::packagedElement/xoev:xsdXModel(.)"/>
                            <xsl:variable name="import-prefix" select="(xoev:xsdImport(.)/@prefix, $imported-package/xoev:xsdSchema(.)/@prefix, $xmodel-of-imported-package/@prefix)[1]"/>
                            <xsl:variable name="imported-namespace" select="($imported-package/xoev:xsdSchema(.)/@namespace, $xmodel-of-imported-package/@namespace)[1]"/>
                            <import id="{$imported-package/@xmi:id}" prefix="{$import-prefix}" namespace="{$imported-namespace}" schemaFile="{xoev:xsdSchema($imported-package)/@schemaFile}" kind="direct">
                                <xsl:choose>
                                    <xsl:when test="$xoev:my-xmodel/xoev:xsdXModel(.)/@deployment = 'true'">
                                        <xsl:attribute name="schemaLocation" select="
                                                (xoev:xsdImport(.)/@schemaLocationDeployment,
                                                xoev:xsdSchema($imported-package)/@schemaLocation,
                                                concat($xmodel-of-imported-package/@schemaLocationBase, xoev:xsdSchema($imported-package)/@schemaFile))[1]"/>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:attribute name="schemaLocation" select="(xoev:xsdImport(.)/@schemaLocationLocal, xoev:xsdSchema($imported-package)/@schemaFile)[1]"/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </import>
                            <xsl:for-each select="xoev:all-included-packages($imported-package)">
                                <import id="{@xmi:id}" prefix="{$import-prefix}" namespace="{$imported-namespace}" schemaFile="{xoev:xsdSchema(.)/@schemaFile}" kind="indirect"/>
                            </xsl:for-each>
                        </xsl:for-each>
                        <xsl:for-each select="xoev:directly-included-packages(.)">
                            <include id="{@xmi:id}" schemaFile="{xoev:xsdSchema(.)/@schemaFile}" kind="direct"/>
                            <xsl:for-each select="xoev:all-included-packages(.)">
                                <include id="{@xmi:id}" schemaFile="{xoev:xsdSchema(.)/@schemaFile}" kind="indirect"/>
                            </xsl:for-each>
                        </xsl:for-each>
                    </package-with-imports-and-includes>

                </xsl:for-each>
            </packages-with-imports-and-includes>
        </xsl:document>
    </xsl:variable>
    <xd:doc>
        <xd:desc>Key zur Ermittlung eines Elements aus <xd:ref name="xoev:packages-with-imports-and-includes" type="variable"/> anhand der ID des betrachteten Pakets.</xd:desc>
    </xd:doc>
    <xsl:key name="xoev:packages-with-imports-and-includes" match="package-with-imports-and-includes" use="@id"/>
    <xsl:function name="xoev:package-with-imports-and-includes" as="element(package-with-imports-and-includes)">
        <xsl:param name="package" as="element()"/>
        <xsl:sequence select="key('xoev:packages-with-imports-and-includes', $package/@xmi:id, $xoev:packages-with-imports-and-includes)"/>
    </xsl:function>

    <xsl:variable name="xoev:ist-zyklenfrei" as="xs:boolean">
        <xsl:value-of select="
                every $p in $uml:root-model//packagedElement[xoev:xsdSchema(.)]
                    satisfies empty(
                    index-of(
                    xoev:allImportedAndIncludedPackageIDs(
                    $p/xoev:dependencies-by-client(.)[xoev:xsdImport(.) or xoev:xsdInclude(.)]/uml:resolve-supplier(.)/@xmi:id,
                    $p/ancestor::uml:Model),
                    $p/@xmi:id))"/>
    </xsl:variable>

</xsl:stylesheet>
